/********************************/
/*                              */
/*        JEU DE MEMOIRE        */
/*                              */
/********************************/
/*  Programme MEMOIRE.C         */
/********************************/
/*  Auteur: Patrick Leclercq    */
/********************************/
#include <stdio.h>
#include <malloc.h>
#include <graphvga.h>
#include <souris.h>
#include <time.h>
#include <loadpcx.h>


#define MAX_CARTES		7

#define TX_CARTE		64
#define TY_CARTE		44

#define NB_LIGNES		3
#define NB_COLONNES		4

#define DOS_CARTE		0

#define	PAS_ACTION		0
#define NOUVEAU_JEU		1
#define QUITTER_PRG		2
#define SELECTION_CARTE	3


/* MEMORISATION DES IMAGES DES CARTES */
byte *image_carte[MAX_CARTES];

/* MEMORISATION TAILLE DES CARTES */
int taille_carte;

/* MEMORISATION DES CARTES */
int tableau_cartes[NB_LIGNES][NB_COLONNES];

/* LISTE DES IMAGES VISIBLES */
int images_visible[NB_LIGNES][NB_COLONNES];


/* MEMORISATION DE LA CARTE COURANTE */
int ligne_courante;
int colonne_courante;

/* GESTION DES CARTES RETOURNEES */
int presence_une_carte;
int ligne_carte1;
int colonne_carte1;
int ligne_carte2;
int colonne_carte2;
int nb_cartes_decouvertes;



/******************************/
/* ATTENTE DE n MILLISECONDES */
/******************************/
/* La temporisation cesse si le joueur presse sur un bouton de la souris */
void temporisation(int attente)
{
	time_t temps;

	temps = clock()+attente;
	do {
			if (clock()>=temps)   break;
			if (clic_souris()!=0) break;
		} while(1);
}


/*********************************************/
/* RESERVATION DE LA MEMOIRE POUR LES CARTES */
/*********************************************/
void reservation_memoire_cartes(void)
{
  int n;
    
  taille_carte=get_taille_image(TX_CARTE, TY_CARTE);
  for (n=0; n<MAX_CARTES; n++)
    image_carte[n]=malloc(taille_carte);
}


/***************************************/
/* LIBERATION DE LA MEMOIRE DES CARTES */
/***************************************/
void liberation_memoire_cartes(void)
{
  int n;

  for (n=0; n<MAX_CARTES; n++)
    free(image_carte[n]);
}



/************************************/
/* CHARGEMENT DES CARTES EN MEMOIRE */
/************************************/
void charger_cartes (char *nom_fichier)
{
   int n;
   FILE *fichier;

   fichier=fopen(nom_fichier, "rb");						/* OUVERTUDE FICHIER */
   for (n=0; n<MAX_CARTES; n++)								/* BOUCLE DE SAUVEHARDE */
     fread (image_carte[n], taille_carte, 1, fichier);		/* SAUVER IMAGE CARTE */
   fclose (fichier);										/* FERMETURE FICHIER */
}
  

/************************************/
/* AFFICHAGE DE L'IMAGE D'UNE CARTE */
/************************************/  
void affichage_image(int px, int py, int num_carte)
{
  put_image(px, py, image_carte[num_carte]);
}

  
/*********************************/
/* TEST SI UNE CARTE EST VISIBLE */
/*********************************/
/* 0: La carte n'est pas visible */
/* 1: La carte est visible       */
/*********************************/
int test_carte_visible(int ligne, int colonne)
{
  return (images_visible[ligne][colonne]);
}


/*************************************************/
/* ECRITURE DE LA CARTE carte DANS UNE CASE VIDE */
/*************************************************/
void tirage_carte(int carte)
{
  int ligne;
  int colonne;

  /* RECHERCHE D'UNE CASE LIBRE */
  do {
		 ligne=rnd(NB_LIGNES);
		 colonne=rnd(NB_COLONNES);
		 if (tableau_cartes[ligne][colonne]==-1) break;
	  } while (1);

  /* ECRITURE CARTE DANS TABLEAU */
  tableau_cartes[ligne][colonne]=carte;
}


/******************************************/
/* GENERATION ALEATOIRE DU TABLEAU DE JEU */
/******************************************/
void generation_tableau_jeu(void)
{
  int carte;
  int ligne;
  int colonne;

  /* INITIALISATION TABLEAU CARTES  */
  for (ligne=0; ligne<NB_LIGNES; ligne++)
	 for (colonne=0; colonne<NB_COLONNES; colonne++)
		tableau_cartes[ligne][colonne]=-1;

  /* TIRAGE DES CARTES  */
  for (carte=1; carte<=6; carte++)
  {
	 tirage_carte(carte);			/* TIRAGE PREMIERE CARTE */
	 tirage_carte(carte);			/* TIRAGE SECONDE CARTE  */
  }
}


/*************************/
/* AFFICHAGE D'UNE CARTE */
/*************************/
void montrer_carte(int ligne, int colonne)
{
  int image_carte;
  int px;
  int py;

  image_carte=tableau_cartes[ligne][colonne];	/* LECTURE NUMERO IMAGE */
  px=colonne*76+14;								/* CALCUL POSITION PX */
  py=ligne*54+35;								/* CALCUL POSITION PY */
  affichage_image(px, py, image_carte);			/* AFFICHAGE IMAGE */
  images_visible[ligne][colonne]=1;				/* MEMORISATION AFFICHAGE IMAGE */
}


/**************************/
/* EFFACEMENT D'UNE CARTE */
/**************************/
void cacher_carte(int ligne, int colonne)
{
  int px;
  int py;

  px=colonne*76+14;						/* CALCUL PY CARTE */
  py=ligne*54+35;						/* CALCUL PX CARTE */
  affichage_image(px, py, DOS_CARTE);	/* AFFICHAGE IMAGE DOS CARTE */
  images_visible[ligne][colonne]=0;		/* MEMORISATION EFFACEMENT IMAGE */
}


/*******************************/
/* AFFICHAGE DU DOS DES CARTES */
/*******************************/
void aff_ecran_jeu(void)
{
  int ligne;
  int colonne;

  for (ligne=0; ligne<NB_LIGNES; ligne++)
	 for (colonne=0; colonne<NB_COLONNES; colonne++)
		cacher_carte(ligne, colonne);
}


/********************************/
/* PREPARATION D'UN NOUVEAU JEU */
/********************************/
void preparation_nouveau_jeu(void)
{
  nb_cartes_decouvertes=0;
  presence_une_carte=0;
  generation_tableau_jeu();
  aff_ecran_jeu();
}


/**********************************/
/* SELECTION D'UNE PREMIERE CARTE */
/**********************************/
void selection_premiere_carte(void)
{
  ligne_carte1=ligne_courante;
  colonne_carte1=colonne_courante;
  eff_souris();
  montrer_carte (ligne_carte1, colonne_carte1);
  aff_souris();
}


/*********************************/
/* SELECTION D'UNE SECONDE CARTE */
/*********************************/
void selection_seconde_carte(void)
{
  int numero_carte1;
  int numero_carte2;

  ligne_carte2=ligne_courante;									/* LECTURE LIGNE CARTE 2 */
  colonne_carte2=colonne_courante;								/* LECTURE COLONNE CARTE 2 */

  /* AFFICHAGE DE LA SECONDE CARTE */
  eff_souris();													/* EFFACEMENT SOURIS */
  montrer_carte(ligne_carte2, colonne_carte2);					/* AFFICHAGE CARTE 2 */
  aff_souris();													/* AFFICHAGE SOURIS  */

  /* LECTURE DES NUMEROS DES CARTES */
  numero_carte1=tableau_cartes[ligne_carte1][colonne_carte1];	/* LECTURE NUMERO CARTE 1 */
  numero_carte2=tableau_cartes[ligne_carte2][colonne_carte2];	/* LECTURE NUMERO CARTE 2 */

  /* LES DEUX CARTES SONT LES MEMES */
  if (numero_carte1==numero_carte2)
  {
	  presence_une_carte=0;								/* REMISE A ZERO DU COMPTEUR */
	  nb_cartes_decouvertes++;							/* MEMORISATION NB PAIRES VISIBLES */
	}
	else

	/* LES DEUX CARTES NE SONT PAS LES MEMES */
	{
	  presence_une_carte=0;								/* REMISE A ZERO DU COMPTEUR */
	  temporisation(800);								/* TEMPORISATION 0,8 SECONDE */
	  eff_souris();										/* EFFACEMENT SOURIS */
	  cacher_carte(ligne_carte1, colonne_carte1);		/* CACHER CARTE 1 */
	  cacher_carte(ligne_carte2, colonne_carte2);		/* CACHER CARTE 2 */
	  aff_souris();										/* AFFICHAGE SOURIS */
	}
}


/**********************/
/* CLIC SUR UNE CARTE */
/**********************/
void clic_sur_carte(void)
{
  if (!test_carte_visible(ligne_courante, colonne_courante))	/* TEST SI CARTE VISIBLE */
	 if (presence_une_carte==0)		/* TEST SI UNE CARTE EST DEJA SUR L'ECRAN */
	 {
		selection_premiere_carte();	/* CLIC SUR LA PREMIERE CARTE */
		presence_une_carte=1;			/* MEMORISATION PRESENCE CARTE */
	 }
	 else
		selection_seconde_carte();	/* CLIC SUR LA SECONDE CARTE */
}


/*********************************/
/* EXECUTION DE L'ACTION CHOISIE */
/* PAR LE JOUEUR                 */
/*********************************/
void traitement_action (int action)
{
  switch (action)
  {
	 case SELECTION_CARTE:
		clic_sur_carte();
		break;

	 case NOUVEAU_JEU:
		eff_souris();
		preparation_nouveau_jeu();
		aff_souris();
		break;
  }
}


/******************************/
/* IDENTIFICATION DE LA CARTE */
/* SELECTIONNEE PAR LE JOUEUR */
/******************************/
int identification_carte (int xm, int ym)
{
  int ligne;
  int colonne;
  int px, py;
  int presence_carte;

  presence_carte=0;											/* INIT INDICATEUR PRESENCE CARTE */
  for (ligne=0; ligne<NB_LIGNES; ligne++)					/* BOUCLE LIGNE */
	 for (colonne=0; colonne<NB_COLONNES; colonne++)			/* BOUCLE COLONNE */
	 {
		px=colonne*76+14;										/* CALCUL PX CARTE */
		py=ligne*54+35;										/* CALCUL PY CARTE */
		if (tst_zone(xm, ym, px, py, TX_CARTE, TY_CARTE))		/* TEST PRESENCE CARTE */
		{
		  ligne_courante=ligne;								/* MEMORISATION LIGNE CARTE */
		  colonne_courante=colonne;							/* MEMORISATION COLONNE CARTE */
		  presence_carte=1;									/* MARQUAGE DETECTION CARTE */
		}
	 }

	return (presence_carte);								/* PARAMETRE DE FIN DU LOGICIEL */
}


/*********************************/
/* ANALYSE DE L'ACTION DU JOUEUR */
/*********************************/
int analyse_action(int xm, int ym)
{
  int reponse;

  reponse=PAS_ACTION;						/* INITIALISATION REPONSE PAR DEFAUT */
  if (tst_zone(xm, ym, 51, 10, 66, 15))		/* TEST SI ZONE "NOUVEAU JEU" */
	 reponse=NOUVEAU_JEU;
  else

  if (tst_zone(xm, ym, 201, 10, 66, 15))	/* TEST SI ZONE "QUITTER PRG" */
	 reponse=QUITTER_PRG;
  else

  if (identification_carte(xm, ym))			/* TEST SI CLIC SUR UNE CARTE */
	 reponse=SELECTION_CARTE;

  return (reponse);							/* VALEUR DU PARAMETRE DE RETOUR */
}


/******************/
/* GESTION DU JEU */
/******************/
void gestion_jeu(void)
{
  int xm, ym, km;
  int action;

  do {
		 att_clic (&xm, &ym, &km);			/* ATTENTE CLIC SOURIS */
		 action=analyse_action(xm, ym);	/* DETERMINATION DE L'ACTION */

		 if (action==QUITTER_PRG)			/* TEST SI FIN DU PROGRAMME */
			break;

		 if (action!=PAS_ACTION)			/* TEST SI ACTION A EXECUTER */
			traitement_action(action);

		} while(1);
}


/***********************/
/* FONCTION PRINCIPALE */
/***********************/
void main(void)
{
  randomize();					/* INITIALISATION GENERATEUR ALEATOIRE */
  init_vga();					/* INITIALISATION MODE GRAPHIQUE VGA */
  reservation_memoire_cartes();
  
  init_souris();				/* INITIALISATION SOURIS */
  eff_souris();					/* EFFACEMENT SOURIS */

  charger_cartes("cartes.img");	/* CHARGEMENT DES CARTES */
  load_pcx("fond.pcx");			/* CHARGEMENT IMAGE DE FOND */
                        	
  preparation_nouveau_jeu();	/* INITIALISATION DU JEU */

  aff_souris();					/* AFFICHAGE SOURIS */
  gestion_jeu();				/* APPEL ROUTINE PRINCIPALE */
  eff_souris();					/* EFFACEMENT SOURIS */
  
  liberation_memoire_cartes();
  fermeture_vga();				/* DESACTIVATION DU MODE GRAPHIQUE */
}

