/**************************************/
/*                                    */
/*    COLUMNS - JEU DE TYPE TETRIS    */
/*                                    */
/**************************************/
/*  Programme COLUMNS.C               */
/**************************************/
/*  Auteur: Patrick Leclercq          */
/**************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>

#include <graphvga.h>


#define NB_COULEURS             4
#define HAUTEUR_BARRE           3

#define EPAISSEUR_CADRE         3
#define COULEUR_CADRE           4
#define COULEUR_FOND            3

#define NB_LIGNES               12
#define NB_COLONNES             7
#define COULEUR_ESPACE_JEU      7

#define TX_CUBE                 8
#define TY_CUBE                 8

#define PX_ESPACE_JEU           100
#define PY_ESPACE_JEU           50


/************************************************/
/* CONSTANTES POUR L'ANALYSE DES ORDRES CLAVIER */
/************************************************/
#define PAS_ACTION              0
#define QUITTER_JEU             1
#define ALLER_GAUCHE            2
#define ALLER_DROITE            3
#define PERMUTATION_COULEURS    4
#define CHUTE_BARRE             5


int score;
int fin_programme;
int vitesse_de_chute;

int  liste_cube[HAUTEUR_BARRE];
int  ligne_barre;
int  colonne_barre;


/***************************************/
/* MEMORISATION DE L'EMPILAGE DE CUBES */
/***************************************/
int tableau_jeu[NB_LIGNES][NB_COLONNES];
int tableau_alignement[NB_LIGNES][NB_COLONNES];


void init_tableau_jeu(void)
{
  int ligne;
  int colonne;

  for (ligne=0; ligne<NB_LIGNES; ligne++)
	 for (colonne=0; colonne<NB_COLONNES; colonne++)
		tableau_jeu[ligne][colonne]=-1;
}


/**********************************/
/* GESTION DE LA VITESSE DE CHUTE */
/**********************************/
void gestion_vitesse_de_chute(void)
{
  if (score<100)   	              vitesse_de_chute=500;
  if ((score>100) && (score<=200)) vitesse_de_chute=300;
  if ((score>200) && (score<=300)) vitesse_de_chute=200;
  if ((score>300) && (score<=400)) vitesse_de_chute=100;
  if (score>400)                   vitesse_de_chute=50;
}


/**********************/
/* AFFICHAGE DU SCORE */
/**********************/
void affichage_score(void)
{
  gotoxy(11, 3);
  printf (" Score: %d ",score);
}


/****************************/
/* DESSIN DE L'ECRAN DU JEU */
/****************************/
void dessin_ecran_jeu(void)
{
  int tx_rectangle;
  int ty_rectangle;
  int px_cadre, py_cadre;
  int tx_cadre, ty_cadre;

  /* SELECTION DE LA COULEUR DE L'ECRAN */
  rectangle (0, 0, 320, 200, COULEUR_FOND);

  /* CALCUL DIMENSION DE L'ESPACE DE JEU */
  tx_rectangle=NB_COLONNES*TX_CUBE;
  ty_rectangle=NB_LIGNES*TY_CUBE;

  /* CALCUL DIMENSIN DU CADRE ENTOURANT L'ESPACE DE JEU */
  px_cadre=PX_ESPACE_JEU-EPAISSEUR_CADRE;
  py_cadre=PY_ESPACE_JEU-EPAISSEUR_CADRE;
  tx_cadre=tx_rectangle+EPAISSEUR_CADRE*2;
  ty_cadre=ty_rectangle+EPAISSEUR_CADRE*2;

  /* AFFICHAGE CADRE */
  rectangle (px_cadre, py_cadre, tx_cadre, ty_cadre, COULEUR_CADRE);
  /* AFFICHAGE DE L'ESPACE DE JEU */
  rectangle (PX_ESPACE_JEU, PY_ESPACE_JEU, tx_rectangle, ty_rectangle, COULEUR_ESPACE_JEU);

  affichage_score();
}


/*************************/
/*  AFFICHAGE D'UN CUBE  */
/*************************/
void affichage_cube (int ligne, int colonne)
{
  int px, py;
  int couleur_cube;

  px=PX_ESPACE_JEU+colonne*TX_CUBE;
  py=PY_ESPACE_JEU+ligne*TY_CUBE;

  couleur_cube=tableau_jeu[ligne][colonne];
  if (couleur_cube!=-1)
	 rectangle (px, py, TX_CUBE, TY_CUBE, couleur_cube);
  else
	 rectangle (px, py, TX_CUBE, TY_CUBE, COULEUR_ESPACE_JEU);
}


int detection_vertical_colonne (int colonne)
{ 
  int ligne;
  int couleur;
  int compteur_couleur;
  int couleur_precedente;
  int presence_alignement;
  int p;

  presence_alignement=0;
  couleur_precedente=-1; /* COULEUR BIDON */
  compteur_couleur=0;
  for (ligne=0; ligne<NB_LIGNES; ligne++)
  {
    couleur=tableau_jeu[ligne][colonne];
    /* VERIFICATION SI LA CASE EST VIDE */
    if (couleur==-1)
      {
			compteur_couleur=0;
			couleur_precedente=-1;
			goto fin;
		}

	 /* DETECTION D'UNE PREMIERE CASE DE COULEUR */
	 if (compteur_couleur==0)
	 {
		 compteur_couleur=1;
		 couleur_precedente=couleur;
		 goto fin;
	 }

	 /* DETECTION D'UNE SEQUENCE DE COULEUR */
	 if (couleur==couleur_precedente)
	 {
		 compteur_couleur++;
		 if (compteur_couleur>=HAUTEUR_BARRE)
		 {
			/* MARQUAGE EN MEMOIRE DE LA SEQUENCE DE COULEUR */
			for (p=0; p<compteur_couleur; p++)
				tableau_alignement[ligne-p][colonne]=1;
			presence_alignement=1;
		 }
		 goto fin;
	 }

	 /* LA SEQUENCE DE COULEUR SE TERMINE */
	 /* DEBUT D'UNE NOUVELLE SEQUENCE */
	 if (couleur!=couleur_precedente)
	 {
		 couleur_precedente=couleur;
		 compteur_couleur=1;
		 goto fin;
	 }

	 fin: ;
  }

  return (presence_alignement);
}


/**************************************/
/* DETECTION D'UN ALIGNEMENT VERTICAL */
/**************************************/
int detection_alignements_verticaux(void)
{
  int presence_alignement;
  int colonne;
			    
  presence_alignement=0;                            
  for (colonne=0; colonne<NB_COLONNES; colonne++)
    if (detection_vertical_colonne(colonne)==1)
      presence_alignement=1;

  return (presence_alignement);
}
  

int detection_horizontal_ligne (int ligne)
{
  int couleur;
  int compteur_couleur;
  int couleur_precedente;
  int presence_alignement;
  int position;
  int p;

  presence_alignement=0;
  couleur_precedente=-1; /* COULEUR BIDON */
  compteur_couleur=0;
  for (position=0; position<NB_COLONNES; position++)
  {
	 couleur=tableau_jeu[ligne][position];
	 /* VERIFICATION SI LA CASE EST VIDE */
	 if (couleur==-1)
		{
			compteur_couleur=0;
			couleur_precedente=-1;
			goto fin;
		}


	 /* DETECTION D'UNE PREMIERE CASE DE COULEUR */
	 if (compteur_couleur==0)
	 {
		 compteur_couleur=1;
		 couleur_precedente=couleur;
		 goto fin;
	 }

	 /* DETECTION D'UNE SEQUENCE DE COULEUR */
	 if (couleur==couleur_precedente)
	 {
		 compteur_couleur++;
		 if (compteur_couleur>=HAUTEUR_BARRE)
		 {
			/* MARQUAGE EN MEMOIRE DE LA SEQUENCE DE COULEUR */
			for (p=0; p<compteur_couleur; p++)
				tableau_alignement[ligne][position-p]=1;
			presence_alignement=1;
			}
			goto fin;
	 }

	 /* LA SEQUENCE DE COULEUR SE TERMINE */
	 /* DEBUT D'UNE NOUVELLE SEQUENCE */
	 if (couleur!=couleur_precedente)
	 {
		 couleur_precedente=couleur;
		 compteur_couleur=1;
		 goto fin;
	 }

	 fin: ;
  }

  return (presence_alignement);
}


/*****************************************/
/* DETECTION DES ALIGNEMENTS HORIZONTAUX */
/*****************************************/
int detection_alignements_horizontaux(void)
{
  int presence_alignement;
  int ligne;

  presence_alignement=0;
  for (ligne=0; ligne<NB_LIGNES; ligne++)
    if (detection_horizontal_ligne(ligne)==1)
		presence_alignement=1;

  return (presence_alignement);
}


/*****************************************/
/* DETECTION D'UN ALIGNEMENT DE COULEURS */
/* DANS UNE LIGNE DIAGONALE              */
/*****************************************/
int detection_ligne_diagonale1 (int ligne, int colonne)
{
  int couleur;
  int compteur_couleur;
  int couleur_precedente;
  int presence_alignement;
  int p;

  presence_alignement=0;
  couleur_precedente=-1; /* COULEUR BIDON */
  compteur_couleur=0;

  do {
		 couleur=tableau_jeu[ligne][colonne];
		 /* VERIFICATION SI LA CASE EST VIDE */
		 if (couleur==-1)
		 {
			compteur_couleur=0;
			couleur_precedente=-1;
			goto fin;
		 }

		 /* DETECTION D'UNE PREMIERE CASE DE COULEUR */
		 if (compteur_couleur==0)
		 {
			compteur_couleur=1;
			couleur_precedente=couleur;
			goto fin;
		 }

		 /* DETECTION D'UNE SEQUENCE DE COULEUR */
		 if (couleur==couleur_precedente)
		 {
			compteur_couleur++;
			if (compteur_couleur>=HAUTEUR_BARRE)
			{
				/* MARQUAGE EN MEMOIRE DE LA SEQUENCE DE COULEUR */
				for (p=0; p<compteur_couleur; p++)
					tableau_alignement[ligne-p][colonne-p]=1;
				presence_alignement=1;
			}
		 goto fin;
		 }

		 /* LA SEQUENCE DE COULEUR SE TERMINE */
		 /* DEBUT D'UNE NOUVELLE SEQUENCE     */
		 if (couleur!=couleur_precedente)
		 {
			couleur_precedente=couleur;
			compteur_couleur=1;
			goto fin;
		 }

		 fin:
		 /* PASSAGE A LA CASE SUIVANTE */
		 ligne++;
	colonne++;

		 /* TEST SI CONDITION DE SORTIE */
		 if ((ligne==NB_LIGNES) | (colonne==NB_COLONNES)) break;
	} while (1);

  return (presence_alignement);
}


/****************************************/
/* DETECTION D'UN ALIGNEMENT HORIZONTAL */
/* A PARTIR D'UNE POSITION              */
/****************************************/
int detection_alignements_diagonal1(void)
{
  int presence_alignement;
  int ligne;
  int colonne;

  presence_alignement=0;
  colonne=0;
  for (ligne=0; ligne<NB_LIGNES; ligne++)
	 if (detection_ligne_diagonale1(ligne, colonne)==1)
		presence_alignement=1;

  ligne=0;
  for (colonne=0; colonne<NB_COLONNES; colonne++)
	 if (detection_ligne_diagonale1(ligne, colonne)==1)
		presence_alignement=1;

  return (presence_alignement);
}


/*******************************************/
/* DETECTION D'UN ALIGNEMENT DE COULEURS   */
/* DANS UNE LIGNE DIAGONALE ORIENTEE A 45 */
/*******************************************/
int detection_ligne_diagonale2 (int ligne, int colonne)
{
  int couleur;
  int compteur_couleur;
  int couleur_precedente;
  int presence_alignement;
  int p;

  presence_alignement=0;
  couleur_precedente=-1; /* COULEUR BIDON */
  compteur_couleur=0;

  do {
		 couleur=tableau_jeu[ligne][colonne];
		 /* VERIFICATION SI LA CASE EST VIDE */
		 if (couleur==-1)
		 {
			compteur_couleur=0;
			couleur_precedente=-1;
			goto fin;
		 }

		 /* DETECTION D'UNE PREMIERE CASE DE COULEUR */
		 if (compteur_couleur==0)
		 {
			compteur_couleur=1;
			couleur_precedente=couleur;
			goto fin;
		 }

		 /* DETECTION D'UNE SEQUENCE DE COULEUR */
		 if (couleur==couleur_precedente)
		 {
			compteur_couleur++;
			if (compteur_couleur>=HAUTEUR_BARRE)
			{
				/* MARQUAGE EN MEMOIRE DE LA SEQUENCE DE COULEUR */
				for (p=0; p<compteur_couleur; p++)
					tableau_alignement[ligne-p][colonne+p]=1;
				presence_alignement=1;
			}
			goto fin;
		 }

		 /* LA SEQUENCE DE COULEUR SE TERMINE */
		 /* DEBUT D'UNE NOUVELLE SEQUENCE     */
		 if (couleur!=couleur_precedente)
		 {
			couleur_precedente=couleur;
			compteur_couleur=1;
			goto fin;
		 }

		 fin:
		/* PASSAGE A LA CASE SUIVANTE */
		ligne++;
		colonne--;

		/* TEST SI CONDITION DE SORTIE */
		if ((ligne==NB_LIGNES) | (colonne<0)) break;
  } while (1);

  return (presence_alignement);
}


/****************************************/
/* DETECTION D'UN ALIGNEMENT HORIZONTAL */
/* A PARTIR D'UNE POSITION              */
/****************************************/
int detection_alignements_diagonal2(void)
{
  int presence_alignement;
  int ligne;
  int colonne;

  presence_alignement=0;
  colonne=7;
  for (ligne=0; ligne<NB_LIGNES; ligne++)
	 if (detection_ligne_diagonale2(ligne, colonne)==1)
		presence_alignement=1;

  ligne=0;
  for (colonne=0; colonne<NB_COLONNES; colonne++)
	 if (detection_ligne_diagonale2(ligne, colonne)==1)
		presence_alignement=1;

  return (presence_alignement);
}


/*********************************************/
/* INITIALISATION DU TABLEAU DES ALIGNEMENTS */
/*********************************************/
void init_tableau_alignement(void)
{
  int ligne;
  int colonne;

  for (ligne=0; ligne<NB_LIGNES; ligne++)
	 for (colonne=0; colonne<NB_COLONNES; colonne++)
		tableau_alignement[ligne][colonne]=-1;
}


/***********************************************/
/* ROUTINE TESTANT LA PRESENCE DES ALIGNEMENTS */
/***********************************************/
int detection_alignements (void)
{
  int presence_alignement;

  init_tableau_alignement();

  presence_alignement=0;
  if (detection_alignements_verticaux ()==1)
	 presence_alignement=1;
  if (detection_alignements_horizontaux()==1)
	 presence_alignement=1;
  if (detection_alignements_diagonal1()==1)
	 presence_alignement=1;
  if (detection_alignements_diagonal2 ()==1)
	 presence_alignement=1;

  return (presence_alignement);
}


/******************************/
/* EFFACEMENT DES ALIGNEMENTS */
/******************************/
void effacement_alignements(void)
{
  int ligne;
  int colonne;

  for (ligne=0; ligne<NB_LIGNES; ligne++)
	 for (colonne=0; colonne<NB_COLONNES; colonne++)
		if (tableau_alignement[ligne][colonne]!=-1)
		{
			/* COMPTABILISATION DU SCORE */
			score++;
			/* EFFACEMENT DU CUBE EN MEMOIRE */
			tableau_jeu[ligne][colonne]=-1;
			/* EFFACEMENT DU CUBE DE L'ECRAN */
			affichage_cube (ligne, colonne);
		}
}


/*********************************/
/* GESTION DE LA CHUTE D'UN CUBE */
/*********************************/
void chute_un_cube (int ligne, int colonne)
{
  int cube;

  do {
		 cube=tableau_jeu[ligne][colonne];
		 /* LE CUBE TOMBE D'UNE CASE VERS LE BAS */
		 tableau_jeu[ligne+1][colonne]=cube;
		 tableau_jeu[ligne][colonne]=-1;
		 affichage_cube(ligne, colonne);
		 affichage_cube(ligne+1, colonne);
		 /* LA CHUTE DOIT S'ARRETER SUR LA DERNIERE LIGNE */
		 if (ligne>=NB_LIGNES-2) break;
		 ligne=ligne+1;
		 /* TEST SI LE CUBE DOIT ENCORE TOMBER D'UNE CASE */
		 if (tableau_jeu[ligne+1][colonne]!=-1) break;
	  } while (1);
}


/*******************/
/* CHUTE DES CUBES */
/*******************/
/*
Cette routine fait tomber vers le bas les cubes
qui ne sont plus soutenus par les cubes effacs.
*/
void chute_des_cubes(void)
{
  int ligne;
  int colonne;

  for (ligne=NB_LIGNES-2; ligne>=0; ligne--)
	 for (colonne=0; colonne<NB_COLONNES; colonne++)
		/* TEST SI LA CASE EN DESSOUS DU CUBE EST VIDE */
		if (tableau_jeu[ligne+1][colonne]==-1)
	chute_un_cube(ligne, colonne);
}


/***************************/
/* GESTION DES ALIGNEMENTS */
/***************************/
void disparition_alignements(void)
{
  int presence_alignement;

  presence_alignement=0;
  /* LA BOUCLE S'EXECUTE TANT QUE LE SYSTEME DETECTE DES ALIGNEMENTS */
  do {
		 presence_alignement=detection_alignements();

		 /* TEST SI ABSENCE ALIGNEMENT */
		 if (presence_alignement==0) break;

		 /* EFFACEMENT DES CUBES ALIGNES */
		 effacement_alignements();

		 /* CHUTE DES CUBES */
		 chute_des_cubes();
	  } while (1);
}


void affichage_barre (void)
{
  int px, py;
  int type_cube;
  int i;

  /* CALCUL DE LA POSITION DE LA BARRE */
  px=PX_ESPACE_JEU+colonne_barre*TX_CUBE;
  py=PY_ESPACE_JEU+ligne_barre*TY_CUBE;

  /* AFFICHAGE DES 3 CUBES DE COULEURS */
  for (i=0; i<HAUTEUR_BARRE; i++)
  {
	 type_cube=liste_cube[i];
	 rectangle (px, py, TX_CUBE, TY_CUBE, type_cube);
	 py+=TY_CUBE;
  }
}


void disparition_barre (void)
{
  int px, py;

  /* CALCUL POSITION BARRE */
  px=PX_ESPACE_JEU+colonne_barre*TX_CUBE;
  py=PY_ESPACE_JEU+ligne_barre*TY_CUBE;

  /* EFFACEMENT BARRE */
  rectangle (px, py, TX_CUBE, HAUTEUR_BARRE*TY_CUBE, COULEUR_ESPACE_JEU);
}


/**************************************/
/* MEMORISATION DES CUBES D'UNE BARRE */
/**************************************/
void memorisation_barre(void)
{
  int ligne;
  int colonne;
  int i;

  ligne=ligne_barre;
  colonne=colonne_barre;
  for (i=0; i<HAUTEUR_BARRE; i++)
  {
	 tableau_jeu[ligne][colonne]=liste_cube[i];
	 ligne++;
  }
}


int test_espace_barre(int ligne, int colonne)
{
  int reponse;
  int i;

  reponse=1;
  for (i=0; i<HAUTEUR_BARRE; i++)
	 if (tableau_jeu[ligne+i][colonne]!=-1)
		reponse=0;
  return (reponse);
}


/***********************************/
/* GENERATION D'UNE NOUVELLE BARRE */
/***********************************/
void generation_nouvelle_barre(void)
{
  /* CALCUL DE LA POSITION DE LA BARRE */
  ligne_barre=0;
  colonne_barre=rnd(NB_COLONNES);

  /* CALCUL DES CUBES CONSTITUANT LA BARRE */
  liste_cube[0]=rnd(NB_COULEURS)+1;
  liste_cube[1]=rnd(NB_COULEURS)+1;
  liste_cube[2]=rnd(NB_COULEURS)+1;

  affichage_barre();

  /* Y A T-IL DE L'ESPACE POUR AFFICHER LA NOUVELLE BARRE ? */
  /* SI NON, C'EST LA FIN DE LA PARTIE  */
  if (test_espace_barre(ligne_barre, colonne_barre)==0)
	  fin_programme=1;

  /* AFFICHAGE DU SCORE */
  affichage_score();
}


/***************************************************/
/*   TEST SI LE DEPLACEMENT EST POSSIBLE A GAUCHE  */
/***************************************************/
/* Rponse: 1 => Le dplacement est possible       */
/*          0 => Le dplacement n'est pas possible */
/***************************************************/
int test_deplacement_gauche(void)
{
  int reponse;

  reponse=1;
  /* LA BARRE EST-ELLE SUR LA PREMIERE COLONNE ? */
  if (colonne_barre==0)
	 reponse=0;
  else
  /* YA T-IL UN OU PLUSIEURS CUBES QUI EMPECHENT LE DEPLACEMENT ? */
  if (test_espace_barre(ligne_barre, colonne_barre-1)==0)
		reponse=0;
  return (reponse);
}


void deplacement_gauche(void)
{
  if (test_deplacement_gauche()==1)
  {
	 disparition_barre();
	 colonne_barre--;
	 affichage_barre();
  }
}


/***************************************************/
/*  TEST SI LE DEPLACEMENT EST POSSIBLE A DROITE   */
/***************************************************/
/* Rponse: 1 => Le dplacement est possible       */
/*          0 => Le dplacement n'est pas possible */
/***************************************************/
int test_deplacement_droite(void)
{
  int reponse;

  reponse=1;
  /* LA BARRE EST-ELLE SUR LA DERNIERE COLONNE ? */
  if (colonne_barre==(NB_COLONNES-1))
	 reponse=0;
  else
  /* Y A T-IL UN OU PLUSIEURS CUBES QUI EMPECHENT LE DEPLACEMENT ? */
  if (test_espace_barre(ligne_barre, colonne_barre+1)==0)
		reponse=0;
  return (reponse);
}


void deplacement_droite(void)
{
  if (test_deplacement_droite()==1)
  {
	 disparition_barre();
	 colonne_barre++;
	 affichage_barre();
  }
}


/*********************************************/
/* TEST SI LA CHUTE DE LA BARRE EST POSSIBLE */
/*********************************************/
/* Retour: 1 => la chute est possible        */
/*         0 => La chute n'est pas possible  */
/*********************************************/
int test_deplacement_bas(void)
{
  int reponse;

  reponse=1;
  /* LA BARRE EST-ELLE EN BAS DU TABLEAU ? */
  if (ligne_barre==(NB_LIGNES-HAUTEUR_BARRE))
	 reponse=0;
  else
  /* Y A T-IL UN CUBE QUI BLOQUE LA BARRE ? */
  if (tableau_jeu[ligne_barre+HAUTEUR_BARRE][colonne_barre]!=-1)
	 reponse=0;
  return (reponse);
}


void deplacement_bas(void)
{
  /* TEST SI LE DEPLACEMENT EST POSSIBLE */
  if (test_deplacement_bas()==1)
  {
	 /* DEPLACEMENT D'UNE LIGNE VERS LE BAS */
	 disparition_barre();
	 ligne_barre++;
	 affichage_barre();
  }
  else
  {
	 /* LA BARRE NE PEUT PLUS ALLER PLUS BAS */
	 /* GENERATION D'UNE AUTRE BARRE */
	 memorisation_barre();
	 disparition_alignements();
	 generation_nouvelle_barre();
  }
}


/****************************************************/
/* CHUTE BRUTALE DE LA BARRE VERS LE BAS DE L'ECRAN */
/****************************************************/
void chute_barre(void)
{
  do {
		 if (test_deplacement_bas()==1)
		 {
			/* DEPLACEMENT D'UNE LIGNE VERS LE BAS */
			disparition_barre();
			ligne_barre++;
			affichage_barre();
		 }
		 else
		 {
			/* CREATION D'UNE NOUVELLE BARRE */
			memorisation_barre();
			disparition_alignements();
			generation_nouvelle_barre();
			/* SORTIE DE LA BOUCLE DE CHUTE */
			break;
		 }

  } while (1);
}


void permutation_couleurs(void)
{
  int i;
  int copie_cube[HAUTEUR_BARRE];

  /* COPIE DES CUBES */
  for (i=0; i<HAUTEUR_BARRE; i++)
	 copie_cube[i]=liste_cube[i];

  /* PERMUTATION DES COULEURS DES CUBES */
  liste_cube[0]=copie_cube[2];
  liste_cube[1]=copie_cube[0];
  liste_cube[2]=copie_cube[1];
  affichage_barre();
}


int analyse_ordre(int code_clavier)
{
  int reponse;

  reponse=PAS_ACTION;
  switch (code_clavier)
  {
	 case 27:
		reponse=QUITTER_JEU;          break;
	 case 0x048:
	 case '8':
	 case 0x04C:
	 case '5':
		reponse=PERMUTATION_COULEURS; break;
	 case 0x04B:
	 case '4':
		reponse= ALLER_GAUCHE;        break;
	 case 0x04D:
	 case '6':
		reponse = ALLER_DROITE;       break;
	 case 0x050:
	 case '2':
		reponse = CHUTE_BARRE;        break;
  }
  return (reponse);
}


/*****************************/
/* ROUTINE PRINCIPALE DU JEU */
/*****************************/
void exec_jeu(void)
{
  int    c;
  int    ordre;
  time_t temps_action;

  temps_action= clock() + vitesse_de_chute;
  do {

		 /* GESTION DU TIMING */
		 if (clock()>=temps_action)
		 {
			deplacement_bas();
			temps_action= clock() + vitesse_de_chute;
		 }

		 /* TEST SI PRESSION SUR LE CLAVIER */
		 if (kbhit())
		 {
			c = getch();
			if (c=='\0')
			c= getch();

			/* EXECUTION DE L'ORDRE CLAVIER */
			ordre = analyse_ordre(c);
			switch (ordre)
			{
				case QUITTER_JEU:          fin_programme=1;        break;
				case PERMUTATION_COULEURS: permutation_couleurs(); break;
				case ALLER_GAUCHE:         deplacement_gauche();   break;
				case ALLER_DROITE:         deplacement_droite();   break;
				case CHUTE_BARRE:          chute_barre();          break;
			}
		 }

		 /* CONTROLE DE LA VITESSE DE JEU */
		 gestion_vitesse_de_chute();

		 /* CONDITION DE SORTIE */
		 if (fin_programme==1) break;
	  } while (1);
}


void init_jeu(void)
{
  fin_programme=0;
  score=0;
  vitesse_de_chute=500;
}


/******************/
/* GESTION DU JEU */
/******************/
void gestion_jeu(void)
{
  int c;

  do {
		 init_jeu();
		 dessin_ecran_jeu();
		 init_tableau_jeu();
		 generation_nouvelle_barre();
		 exec_jeu();
		 gotoxy(6, 22);
		 printf (" Une autre partie (O/N) ? ");
		 /* ATTENTE DES LETTRES 'O' OU 'N' */
		 do {
		 c =getch();
		 if ((c=='o') | (c=='O')) break;
		 if ((c=='n') | (c=='N')) break;
		} while (1);
		 /* TEST SI FIN DE PARTIE */
		 if ((c=='n') | (c=='N')) break;
		} while (1);
}


void main(void)
{
  randomize();
  init_vga();
  gestion_jeu();
  fermeture_vga();
}

