/******************************************/
/*                                        */
/*          GESTION DE LA SOURIS          */
/*                                        */
/******************************************/
/*  Bibliothque SOURIS.H                 */
/******************************************/
/*  Auteur: Patrick Leclercq              */
/******************************************/ 
#ifndef _SOURIS


#include <DOS.H>


#define NON_CLIC	0
#define CLIC_GAUCHE	1
#define CLIC_DROIT	2
#define CLIC_MILIEU	4
#define CLIC_DOUBLE	3


/****************************/
/* AFFICHAGE CURSEUR SOURIS */
/****************************/
void aff_souris(void)
{
  union REGS reg;

  reg.x.ax=0x01;
  int86(0x33, &reg, &reg);
}


/*****************************/
/* EFFACEMENT CURSEUR SOURIS */
/*****************************/
void eff_souris(void)
{
  union REGS reg;

  reg.x.ax=0x02;
  int86(0x33, &reg, &reg);
}


/*******************************/
/* INITIALISATION DE LA SOURIS */
/*******************************/
int init_souris (void)
{
  union REGS reg;
  int   nb_boutons;

  reg.x.ax=0x00;
  int86(0x33, &reg, &reg);
  nb_boutons=0;
  if (reg.x.ax!=0x00)
  {
	 nb_boutons=reg.h.bl;
	 aff_souris();
  }
  return (nb_boutons);
}


/************************************/
/* LECTURE DES BOUTONS DE LA SOURIS */
/************************************/
int clic_souris(void)
{
  union REGS reg;
  int   clic;

  reg.x.ax=0x03;
  int86(0x33, &reg, &reg);
  clic=reg.h.bl & 3;
  return (clic);
}


/******************************/
/* ATTENTE PAS DE CLIC SOURIS */
/******************************/
void att0clic(void)
{
  do {} while (clic_souris()!=0);
}


/****************************/
/* ATTENTE D'UN CLIC SOURIS */
/****************************/
int wait_clic(void)
{
  int clic;

  do {
		 clic=clic_souris();
	  } while (clic==NON_CLIC);
  att0clic();

  return (clic);
}


/*****************************/
/* LECTURE POSITION X SOURIS */
/*****************************/
int x_souris(void)
{
  union REGS reg;
  int   px;

  reg.x.ax=0x03;
  int86(0x33, &reg, &reg);

  px=reg.x.cx/2;

  return (px);
}


/*****************************/
/* LECTURE POSITION Y SOURIS */
/*****************************/
int y_souris(void)
{
  union REGS reg;
  int   py;

  reg.x.ax=0x03;
  int86(0x33, &reg, &reg);

  py=reg.x.dx;
  return (py);
}


/**********************************/
/* LECTURE DE L'ETAT DE LA SOURIS */
/**********************************/
void souris(int *xm, int *ym, int *km)
{
  union REGS reg;
  int   clic;
  int   px;
  int   py;

  reg.x.ax=0x03;
  int86(0x33, &reg, &reg);

  px=reg.x.cx/2;
  py=reg.x.dx;
  clic=reg.h.bl & 3;

  *xm=px;
  *ym=py;
  *km=clic;
}


/***************************/
/* ATTENTE CLIC            */
/* ET LECTURE ETAT SOURIS  */
/***************************/
void att_clic(int *xm, int *ym, int *km)
{
  union REGS reg;
  int   clic;
  int   px;
  int   py;

  /* ATTENTE CLIC */
  do {
		 reg.x.ax=0x03;
		 int86(0x33, &reg, &reg);

		 px=reg.x.cx/2;
		 py=reg.x.dx;
		 clic=reg.h.bl & 3;
		 if (clic!=0) break;
	  } while (1);

  att0clic();

  *xm=px;
  *ym=py;
  *km=clic;
}


/*************************************/
/* TEST SI UNE POSITION (XM,YM)      */
/* EST INCLUSE DANS UNE ZONE PRECISE */
/*************************************/
int tst_zone(int xm, int ym, int px, int py, int tx, int ty)
{
  int px2, py2;
  int rep;

  px2=px+tx-1;
  py2=py+ty-1;

  rep=0;
  if ((xm>=px) & (xm<=px2) & (ym>=py) & (ym<=py2))
	 rep=1;

  return (rep);
}


/******************************/
/* ATTENTE CLIC DANS UNE ZONE */
/******************************/
void attente_clic_zone(int px, int py, int tx, int ty)
{
  int xm, ym, km;

  do {
		 att_clic(&xm, &ym, &km);
		 if (tst_zone(xm,ym,px,py,tx,ty)==1)
			break;
	  } while (1);
  att0clic();
}


/*******************************/
/* POSITIONNEMENT DE LA SOURIS */
/* A UNE POSITION DE L'ECRAN   */
/*******************************/
void move_souris(int px, int py)
{
  union REGS reg;

  reg.x.ax=0x04;
  reg.x.cx=px*2;
  reg.x.dx=py;
  int86(0x33, &reg, &reg);
}

#define _SOURIS
#endif
