/***********************************************/
/*                                             */
/*    BIBLIOTHEQUE DE FONCTIONS GRAPHIQUES     */
/*       MODE VGA 320x200 - 256 COULEURS       */
/*                                             */
/***********************************************/
/*  Programme GRAPHVGA.H                       */
/***********************************************/
/*  Auteur: Patrick Leclercq                   */
/***********************************************/
#ifndef _GRAPHVGA

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include <malloc.h>
#include <time.h>


typedef unsigned char byte;		/* TYPE DE DONNEES SPECIAL "OCTET" */
typedef unsigned int  uint;


/*********************************/
/* PASSAGE EN MODE GRAPHIQUE VGA */
/*********************************/
void init_vga()
{
  union REGS	reg;
  
  reg.h.ah=0;
  reg.h.al=19;
  int86(0x10, &reg, &reg);
}


/*****************************/
/* FIN DU MODE GRAPHIQUE VGA */
/*****************************/
void fermeture_vga()
{
  union REGS	reg;
  
  reg.h.ah=0;
  reg.h.al=3;
  int86(0x10, &reg, &reg);
}


/*************************/
/* EFFACEMENT DE L'ECRAN */
/*************************/
void cls_ecran(int c)
{
  unsigned char *adr_ecran;			/* CREATION VARIABLE POINTEUR */

  adr_ecran=_MK_FP(0xA000, 0x0);	/* ADRESSE MEMOIRE ECRAN VGA */
  memset (adr_ecran, c, 64000u);	/* REMPLISSAGE DE LA MEMOIRE VIDEO */
}


/*************************/
/* LECTURE COULEUR PIXEL */
/*************************/
byte get_pixel(int px, int py)
{
  byte *adr_pixel;
  byte pixel;
  
  adr_pixel = _MK_FP(0xA000, 0x0);	/* ADRESSE DE LA MEMOIRE ECRAN */
  adr_pixel += py*320+px;			/* CALCUL ADRESSE PIXEL */
  pixel = *adr_pixel;				/* LECTURE PIXEL */
  return (pixel);					/* PARAMETRE DE RETOUR DE LA FONCTION */
}


/**************************/
/* ECRITURE COULEUR PIXEL */
/**************************/
void put_pixel(int px, int py, int couleur)
{
  byte *adr_pixel;
  
  adr_pixel = _MK_FP(0xA000, 0x0);	/* ADRESSE DE LA MEMOIRE ECRAN */
  adr_pixel += py*320+px;			/* CALCUL ADRESSE PIXEL */
  *adr_pixel = couleur;				/* LECTURE PIXEL */
}


/*********************************/
/* TRACE D'UNE LIGNE HORIZONTALE */
/*********************************/
/* px: Position X de la ligne    */
/* py: Position Y de la ligne    */
/* tx: Longueur de la ligne      */
/* c:  Couleur de la ligne       */
/*********************************/
void h_line (int px, int py, int tx, int c)
{
  byte *adr_ligne;

  adr_ligne=_MK_FP(0xA000, 0x0);	/* CALCUL ADRESSE MEMOIRE ECRAN */
  adr_ligne+=py*320+px;				/* CALCUL ADRESSE DEBUT LIGNE */
  memset (adr_ligne, c, tx);		/* REMPLISSAGE LIGNE */
}


/***********************************/
/*   TRACE D'UNE LIGNE VERTICALE   */
/***********************************/
/* px:      Position X de la ligne */
/* py:      Position Y de la ligne */
/* hauteur: Hauteur de la ligne    */
/* c:       Couleur de la ligne    */
/***********************************/
void v_line (int px, int py, int hauteur, int c)
{
  byte *adr_ligne;
  int  n;

  adr_ligne=_MK_FP(0xA000, 0x0);	/* CALCUL ADRESSE MEMOIRE ECRAN */
  adr_ligne+=py*320+px;				/* CALCUL ADRESSE DEBUT LIGNE */
  for (n=0; n<hauteur; n++)			/* BOUCLE D'AFFICHAGE DES PIXELS */
  {
    *adr_ligne=c;					/* AFFICHAGE PIXEL */
    adr_ligne+=320;					/* PASSAGE PIXEL SUIVANT */
  } 
}


/***************************************/
/* AFFICHAGE D'UN RECTANGLE DE COULEUR */
/***************************************/
void rectangle (int px, int py, int tx, int ty, int c)
{
  byte *adr_ligne;
  int  n;

  adr_ligne=_MK_FP(0xA000, 0x0);	/* CALCUL ADRESSE MEMOIRE ECRAN */
  adr_ligne += py*320+px;			/* CALCUL ADRESSE PREMIER LIGNE */
  for (n=0; n<ty; n++)				/* BOUCLE D'AFFICHAGE DES LIGNES */
  {
    memset (adr_ligne, c, tx);		/* AFFICHAGE D'UNE LIGNE */
    adr_ligne+=320;					/* PASSAGE A LA LIGNE SUIVANTE */
  }
}


/************************/
/*   DESSIN D'UN CADRE  */
/************************/
/* px: Position X cadre */
/* py: Position Y cadre */
/* tx: Longueur cadre   */
/* ty: Hauteur cadre    */
/* c:  Couleur cadre    */
/************************/
void cadre (int px, int py, int tx, int ty, int c)
{
  h_line (px, py, tx, c);		/* DESSIN DE LA LIGNE DU HAUT */
  h_line (px, py+ty-1, tx, c);	/* DESSIN DE LA LIGNE DU BAS */
  v_line(px, py, ty, c);		/* DESSIN DE LA LIGNE GAUCHE */
  v_line(px+tx-1, py, ty, c);	/* DESSIN DE LA LIGNE DROITE */
}


/*******************************************/
/*  AFFICHAGE D'UN BLOC                    */
/*******************************************/
/* px: Position X bloc                     */
/* py: Position Y bloc                     */
/* tx: Longueur bloc                       */
/* ty: Hauteur bloc                        */
/* couleur_fond:  Couleur de fond du bloc  */
/* couleur_cadre: Couleur du cadre du bloc */
/*******************************************/
void bloc(int px, int py, int tx, int ty, int couleur_fond, int couleur_cadre)
{
  rectangle(px, py, tx, ty, couleur_fond);
  cadre    (px, py, tx, ty, couleur_cadre);
}


/**************************************/
/* CALCUL DE LA TAILLE MEMOIRE        */
/* NECESSAIRE AU STOCKAGE D'UNE IMAGE */
/**************************************/
unsigned int get_taille_image(int tx, int ty)
{
  return (4+tx*ty);
}


/**************************************/
/* LECTURE DES DIMENSIONS D'UNE IMAGE */
/**************************************/
void get_dimensions_image(byte *adr_image, int *adr_tx, int *adr_ty)
{
  byte *adr;
  
  adr=adr_image;	/* ADRESSE PX */
  *adr_tx=*adr;		/* LECTURE PX */
  adr++;			/* ADRESSE PY */
  *adr_ty=*adr;		/* LECTURE PY */
}


/************************************************/
/*  TRANSFERT D'UNE IMAGE DANS UNE ZONE MEMOIRE */
/************************************************/
/* px: Position X image                         */
/* py: Position Y image                         */
/* tx: Longueur image                           */
/* ty: Hauteur image                            */
/* adr_stockage: Adresse de stockage de l'image */
/************************************************/
void get_image (int px, int py, int tx, int ty, byte *adr_stockage)
{
  byte *adr_image;
  int  *adr_tx;
  int  *adr_ty;
  int  ligne;

  adr_image=_MK_FP(0xA000, 0x0);
  adr_image+=py*320 + px;					/* CALCUL ADRESSE DE L'IMAGE SUR L'ECRAN */
  adr_tx=(int *)adr_stockage;				/* CALCUL ADRESSE DE STOCKAGE DE TX */
  *adr_tx=tx;								/* MEMORISATION DE TX */
  adr_ty=(int *)(adr_stockage+2);			/* CALCUL ADRESSE DE STOCKAGE DE TY */
  *adr_ty=ty;								/* MEMORISATION DE TY */
  
  adr_stockage+=4;							/* CALCUL ADRESSE STOCKAGE DE L'IMAGE */
  for (ligne=0; ligne<ty; ligne++)			/* BOUCLE DE TRANSFERT DES LIGNES */
  {
    memcpy(adr_stockage, adr_image, tx);	/* COPIE D'UNE LIGNE */
    adr_stockage+=tx;						/* CALCUL LIGNE SUIVANTE MEMOIRE */
    adr_image+=320;							/* CALCUL LIGNE SUIVANTE ECRAN */
  }    
}


/************************************************/
/*  TRANSFERT D'UNE IMAGE SUR L'ECRAN           */
/************************************************/
/* px: Position X de l'image                    */
/* py: Position Y de l'image                    */
/* adr_stockage: Adresse de stockage de l'image */
/************************************************/
void put_image (int px, int py, byte *adr_stockage)
{
  byte *adr_image;
  int  *adr_tx;
  int  *adr_ty;
  int  tx, ty;
  int  ligne;
  
  adr_tx=(int *)adr_stockage;				/* CALCUL ADRESSE DE TX (LONGUEUR IMAGE) */
  tx=*adr_tx;								/* LECTURE DE TX */
  adr_ty=(int *)(adr_stockage+2);			/* CALCUL ADRESSE DE TY (HAUTEUR IMAGE) */
  ty=*adr_ty;								/* LECTURE DE TY */

  adr_image = _MK_FP(0xA000, 0x0);			/* CALCUL ADRESSE MEMOIRE ECRAN */
  adr_image += py*320+px;					/* CALCUL ADRESSE IMAGE SUR ECRAN */
  adr_stockage+=4;							/* CALCUL ADRESSE IMAGE DANS MEMOIRE */
  
  for (ligne=0; ligne<ty; ligne++)			/* BOUCLE DE TRANSFERT DES LIGNES */
  {
    memcpy(adr_image, adr_stockage, tx);	/* TRANSFERT LIGNE */
    adr_stockage+=tx;						/* CALCUL LIGNE SUIVANTE DANS MEMOIRE */
    adr_image+=320;							/* CALCUL LIGNE SUIVANTE SUR ECRAN */
  }
}


/***********************************************/
/* COPIE D'UNE IMAGE D'UNE POSITION DE L'ECRAN */
/* VERS UNE AUTRE POSITION DE L'ECRAN          */
/***********************************************/
/* px1: Position X de l'image                  */
/* py1: Position Y de l'image                  */
/* tx:  Longueur image                         */
/* ty:  Hauteur image                          */
/* px2: Position X de copie                    */
/* py2: Position Y de copie                    */
/***********************************************/
void copie_image (int px1, int py1, int tx, int ty, int px2, int py2)
{
  byte *adr_ligne1;
  byte *adr_ligne2;
  int  i;

  adr_ligne1 = _MK_FP(0xA000, 0x0);		/* CALCUL ADRESSE MEMOIRE ECRAN */
  adr_ligne1 += py1*320 + px1;			/* CALCUL ADRESSE IMAGE ORIGINE */
  adr_ligne2 = _MK_FP(0xA000, 0x0);		/* CALCUL ADRESSE MEMOIRE ECRAN */
  adr_ligne2 += py2*320 + px2;			/* CALCUL ADRESSE IMAGE DE DESTINATION */
  for (i=0; i<ty; i++)
  {
    memcpy(adr_ligne2, adr_ligne1, tx);	/* COPIE LIGNE */
    adr_ligne1+=320;					/* LIGNE SUIVANTE IMAGE ORIGINE */
    adr_ligne2+=320;					/* LIGNE SUIVANTE IMAGE DE DESTINATION */
  }
}



/****************************************/
/* DETERMINATION D'UN NOMBRE ALEATOIRE  */
/****************************************/
int rnd (int max_random)
{
  return (rand() % max_random);
}


/******************************************/
/* INITIALISATION DU GENERATEUR ALEATOIRE */
/******************************************/
void randomize()
{
  srand( (unsigned)time( NULL ) );
}


/***********************************/
/* POSITIONNEMENT DU CURSEUR TEXTE */
/***********************************/
void gotoxy(int colonne, int ligne)
{
  union REGS reg;

  reg.h.ah = 2;
  reg.h.bh = 0;
  reg.h.dh = ligne-1;
  reg.h.dl = colonne-1;
  int86(0x10, &reg, &reg);
}


#define _GRAPHVGA
#endif

